#!/usr/bin/perl -w

# Copyright (C) 2010-2015 by Oleksandr Shneyder <oleksandr.shneyder@obviously-nice.de>
# Copyright (C) 2010-2015 by Heinz-M. Graesing <heinz-m.graesing@obviously-nice.de>

# X2Go Admin Server is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.

# X2Go Admin Server is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>
# or write to
# Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

use strict;
use Frontier::RPC2;

use lib "/usr/lib/x2go";
use x2goadminserver;


process_cgi_call({
'x2goadmin.getSessions' => \&x2goadminserver::x2goadmin_getSessions,
'x2goadmin.suspendSession' => \&x2goadminserver::x2goadmin_suspendSession,
'x2goadmin.terminateSession' => \&x2goadminserver::x2goadmin_terminateSession,
'x2goadmin.getGroupsOfUser' => \&x2goadminserver::x2goadmin_getGroupsOfUser,
'x2goadmin.modifyUser' => \&x2goadminserver::x2goadmin_modifyUser,
'x2goadmin.modifyGroup' => \&x2goadminserver::x2goadmin_modifyGroup,
'x2goadmin.addUser' => \&x2goadminserver::x2goadmin_addUser,
'x2goadmin.addGroup' => \&x2goadminserver::x2goadmin_addGroup,
'x2goadmin.removeUsers' => \&x2goadminserver::x2goadmin_removeUsers,
'x2goadmin.removeGroups' => \&x2goadminserver::x2goadmin_removeGroups,
'x2goadmin.getGroupsWithUsers' => \&x2goadminserver::x2goadmin_getGroupsWithUsers,
'x2goadmin.getUsers' => \&x2goadminserver::x2goadmin_getUsers,
'x2goadmin.getGroups' => \&x2goadminserver::x2goadmin_getGroups
}
);

#==========================================================================
#  CGI Support
#==========================================================================
#  Simple CGI support for Frontier::RPC2. You can copy this into your CGI
#  scripts verbatim, or you can package it into a library.
#  (Based on xmlrpc_cgi.c by Eric Kidd <http://xmlrpc-c.sourceforge.net/>.)

# Process a CGI call.
sub process_cgi_call ($) {
	my ($methods) = @_;

	# Get our CGI request information.
	my $method = $ENV{'REQUEST_METHOD'};
	my $type = $ENV{'CONTENT_TYPE'};
	my $length = $ENV{'CONTENT_LENGTH'};

	# Perform some sanity checks.
	http_error(405, "Method Not Allowed") unless $method eq "POST";
	http_error(400, "Bad Request") unless $type eq "text/xml";
	http_error(411, "Length Required") unless $length > 0;

	# Fetch our body.
	my $body;
	my $count = read STDIN, $body, $length;
	http_error(400, "Bad Request") unless $count == $length;

	# Serve our request.
	my $coder = Frontier::RPC2->new;
	send_xml($coder->serve($body, $methods));
}

# Send an HTTP error and exit.
sub http_error ($$) {
	my ($code, $message) = @_;
	print <<"EOD";
Status: $code $message
Content-type: text/html

<title>$code $message</title>
<h1>$code $message</h1>
<p>Unexpected error processing XML-RPC request.</p>
EOD
	exit 0;
}

# Send an XML document (but don't exit).
sub send_xml ($) {
	my ($xml_string) = @_;
	my $length = length($xml_string);
	print <<"EOD";
Status: 200 OK
Content-type: text/xml
Content-length: $length

EOD
    # We want precise control over whitespace here.
	print $xml_string;
}
