# Copyright (C) 2010-2015 by Oleksandr Shneyder <oleksandr.shneyder@obviously-nice.de>
# Copyright (C) 2010-2015 by Heinz-M. Graesing <heinz-m.graesing@obviously-nice.de>

# X2Go Admin Server is free software; you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published
# by the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.

# X2Go Admin Server is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>
# or write to
# Free Software Foundation, Inc.,
# 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.

package x2goadminserver_backend_cipux;
use strict;
use Config::Simple;
use CipUX::RPC::Client;
use English qw( -no_match_vars);


my $Config = new Config::Simple(syntax=>'ini');
$Config->read('/etc/x2go/x2goadminserver.conf' ) or die "/etc/x2go/x2goadminserver.conf";
my $url=$Config->param("cipux_backend.url");

#my $client='rpc_x2go_cipux';
my $client='expl_rpc_task_list';
my $version='0.0.1';

my $perr;
my $rpc=CipUX::RPC::Client->new(
{
	url => $url, client => $client, version => $version,
}
);

use base 'Exporter';
our @EXPORT=('getUsers', 'getGroups', 'getGroupsOfUser', 'getGroupsWithUsers',
             'modifyUser', 'modifyGroup', 'addUser', 'addGroup', 'removeUsers', 'removeGroups');

sub ping
{
	eval { $rpc->rpc_ping;};
	if($EVAL_ERROR)
	{
		$perr=$EVAL_ERROR;
		return 0;
	}
	else
	{
		return 1;
	}
}

sub login_srv
{
	my ($login, $password) = @_;
	my $login_info=
	{
		login => $login,
		password => $password
	};
	return $rpc->rpc_login($login_info);
}

sub getServerInfo
{
	my ($login, $password) = @_;
	if(!ping)
	{
		return (0, "SERVERDOWN URL:$url ERROR:$perr");
	}

	if(!login_srv($login, $password))
	{
		return (0, "NOACCESS");
	}
	return (1,"OK");
}


sub execCommand
{
	my ($cmd,$param,$value)=@_;
	my $msg;
	my $status=1;
	my $a_hr;
	if($param)
	{
		my $p_hr;
		if($value)
		{
			$p_hr={object => $param, value => $value,};
		}
		else
		{
			$p_hr={object => $param,};
		}
			$a_hr= $rpc->xmlrpc( { cmd => $cmd, param_hr => $p_hr} );
	}
	else
	{
		$a_hr= $rpc->xmlrpc( { cmd => $cmd} );
	}
	my $cmdres=$a_hr->{'cmdres_r'};
	if($a_hr->{'status'} eq 'FALSE')
	{
		$status=0;
		$rpc->rpc_logout;
		$msg='CMDERR '.$a_hr->{'cmd'}.' '.$a_hr->{'problem'}.' '.$a_hr->{'msg'};
	}
	return ($status, $cmdres, $msg);
}

sub getRoleGroups
{
	my @groups;
	my ($status,$cmdres, $msg)=execCommand('cipux_task_list_role_accounts');
	if($status)
	{
		while( my ($key, $value) = each (%$cmdres))
		{
			push(@groups, $key);
		}
	}
	return ($status, $msg, @groups);
}

sub changeUser
{
	my ($user,@args)=@_;
	my @newgrp;
	my @obsgrp;
	foreach (@args)
	{
		my @cmd=split(":",$_);
		if(@cmd[0] eq "fname")
		{
			my ($status,$cmdres, $msg)=execCommand('cipux_task_change_user_account_firstname',$user,@cmd[1]);
			if(!$status)
			{
				return $msg;
			}
		}
		if(@cmd[0] eq "lname")
		{
			my ($status,$cmdres, $msg)=execCommand('cipux_task_change_user_account_lastname',$user,@cmd[1]);
			if(!$status)
			{
				return $msg;
			}
		}
		if(@cmd[0] eq "password")
		{
			my ($status,$cmdres, $msg)=execCommand('cipux_task_change_user_account_password',$user,@cmd[1]);
			if(!$status)
			{
				return $msg;
			}
		}
		if(@cmd[0] eq "newgroups")
		{
			@newgrp=split(";",@cmd[1]);
		}
		if(@cmd[0] eq "obsoletegroups")
		{
			@obsgrp=split(";",@cmd[1]);
		}
	}
	if(@obsgrp || @newgrp)
	{
		my ($rstatus,$rmsg,@rolegroups)=getRoleGroups();
		if(!$rstatus)
		{
			return $rmsg;
		}
		foreach(@obsgrp)
		{
			my $grp=$_;
			if( grep { $_ eq $grp} @rolegroups)
			{
				my ($status,$cmdres, $msg)=execCommand('cipux_task_remove_member_from_role_account',$grp,$user);
				if(!$status)
				{
					return $msg;
				}
			}
			else
			{
				my ($status,$cmdres, $msg)=execCommand('cipux_task_remove_member_from_class_share',$grp,$user);
				if(!$status)
				{
					return $msg;
				}
			}
		}
		foreach(@newgrp)
		{
			my $grp=$_;
			if( grep { $_ eq $grp} @rolegroups)
			{
				my ($status,$cmdres, $msg)=execCommand('cipux_task_add_member_to_role_account',$grp,$user);
				if(!$status)
				{
					return $msg;
				}
			}
			else
			{
				my ($status,$cmdres, $msg)=execCommand('cipux_task_add_member_to_class_share',$grp,$user);
				if(!$status)
				{
					return $msg;
				}
			}
		}
	}
	$rpc->rpc_logout;
	return "OK";
}

sub modifyUser
{
	my ($login,$password,$user, @args)=@_;
	my $retmsg="OK";

	my ($status,$str)=getServerInfo($login, $password);
	if(!$status)
	{
		return $str;
	}
	return changeUser($user,@args);
}

sub addUser
{
	my ($login,$password,$user, @args)=@_;
	my $retmsg="OK";

	my ($status,$str)=getServerInfo($login, $password);
	if(!$status)
	{
		return $str;
	}
	my $res;
	my $msg;
	($status,$res, $msg)=execCommand('cipux_task_create_user_account',$user);
	if(!$status)
	{
		return $msg;
	}
	return changeUser($user,@args);
}

sub getUsers
{
	my ($login, $password) = @_;
	my ($status,$str)=getServerInfo($login, $password);
	if(!$status)
	{
		return $str;
	}
	$str=userAttributes();
	my $cmdres;
	my $msg;
	($status,$cmdres, $msg)=execCommand('cipux_task_retrieve_all_user_account_lastname_firstname');
	if(!$status)
	{
		return $msg;
	}
	$str=$str."\nBEGIN_USERS";
	($status,$msg)=getAllUsers();
	if(! $status)
	{
		return $msg;
	}
	$str=$str.$msg."\nEND_USERS";

	$rpc->rpc_logout;
	return $str;
}

sub userAttributes
{
	return "BEGIN_USERATTR\
uid:edit=0;unicode=0\
firstname:edit=1;unicode=0\
lastname:edit=1;unicode=0\
END_USERATTR";
}

sub groupAttributes
{
	return "BEGIN_GROUPATTR\
name:edit=0;unicode=0\
type:edit=0;vars=class_share,course_share,lecture_share,reading_share,seminar_share,studygroup_share,team_share,tutorial_share,workshop_share\
password:edit=1;oncreate=0\
members:edit=1;oncreate=0\
END_GROUPATTR";
}

sub getGroups
{
	my ($login, $password) = @_;
	my ($status,$str)=getServerInfo($login, $password);
	if(!$status)
	{
		return $str;
	}
	$str=groupAttributes();
	my $cmdres;
	my $msg;
	$str=$str."\nBEGIN_USERS";
	($status,$msg)=getAllUsers();
	if(! $status)
	{
		return $msg;
	}
	$str=$str.$msg."\nEND_USERS";
	$str=$str."\nBEGIN_GROUPS\n";
	my $res;
	($status,$msg,$res)=getAllGroupsWithUsers();
	if(!$status)
	{
		return $msg;
	}
	$str=$str.$res;
	$str=$str."\nEND_GROUPS\n";

	$rpc->rpc_logout;
	return $str;
}

sub getAllUsers
{
	my ($status,$cmdres, $msg)=execCommand('cipux_task_retrieve_all_user_account_lastname_firstname');
	if(!$status)
	{
		return  ($status, $msg);
	}
	my $str;
	while( my ($key, $value) = each (%$cmdres))
	{
		if($value->{'cipuxFirstname'}[0] ne "Role" && $value->{'cipuxLastname'}[0] ne "Account")
		{
			$str=$str."\nUID=".$value->{'uid'}[0].":FNAME=".$value->{'cipuxFirstname'}[0].":LNAME=".$value->{'cipuxLastname'}[0];
		}
	}
	return ($status, $str);
}


sub getGroupsWithUsers
{
	my ($login, $password) = @_;
	my ($status,$str)=getServerInfo($login, $password);
	if(!$status)
	{
		return $str;
	}
	$str=userAttributes;
	my $cmdres;
	my $msg;
	$str=$str."\nBEGIN_USERS";
	($status,$msg)=getAllUsers();
	if(! $status)
	{
		return $msg;
	}
	$str=$str.$msg."\nEND_USERS";
	$str=$str."\nBEGIN_GROUPS\n";
	my $res;
	($status,$msg,$res)=getAllGroupsWithUsers();
	if(!$status)
	{
		return $msg;
	}
	$str=$str.$res;
	$str=$str."\nEND_GROUPS\n";

	$rpc->rpc_logout;
	return $str;
}

sub getAllGroupsWithUsers
{

	my $str;
	my ($status,$cmdres_all_tasks, $msg)=execCommand('cipux_task_list_tasks');
	if(!$status)
	{
		return ($status, $msg, $str);
	}
	while( my ($key, $value) = each (%$cmdres_all_tasks))
	{
		if($key =~ m/list/ && $key =~ m/accounts/ &&
		   $key ne "cipux_task_list_user_accounts" &&
		   $key ne "cipux_task_list_role_accounts" &&
		   $key ne "cipux_task_list_skel_accounts")
		{
			my $cmdres;
			($status,$cmdres, $msg)=execCommand($key);
			if(!$status)
			{
				return ($status, $msg, $str);
			}
			while( my ($gkey, $gvalue) = each (%$cmdres))
			{
				my $aref=$gvalue->{'memberUid'};
				$str=$str."\nNAME=".$gkey.":TYPE=role:MEMBERS=".join(";",@$aref);
			}
		}
		if($key =~ m/list/ && $key =~ m/shares/)
		{
			my $cmdres;
			($status,$cmdres, $msg)=execCommand($key);
			if(!$status)
			{
				return ($status, $msg, $str);
			}
			while( my ($gkey, $gvalue) = each (%$cmdres))
			{
				my $sharetype=(split("_",$key))[3];
				my $cmdshareres;
				($status,$cmdshareres, $msg)=execCommand("cipux_task_list_members_of_$sharetype"."_share", $gkey);
				if(!$status)
				{
					return ($status, $msg, $str);
				}
				while( my ($gkey, $gvalue) = each (%$cmdshareres))
				{
					my $aref=$gvalue->{'memberUid'};
					$str=$str."\nNAME=".$gkey.":TYPE=".$sharetype."_share:MEMBERS=".join(";",@$aref);
				}
			}
		}
	}
	return ($status, $msg, $str);
}

sub getGroupsOfUser
{
	my ($login, $password, $user) = @_;
	my ($status,$str)=getServerInfo($login, $password);

	if(!$status)
	{
		return $str;
	}
	$str="";

	my $cmdres_all_tasks;
	my $msg;
	($status,$cmdres_all_tasks, $msg)=execCommand('cipux_task_list_tasks');
	if(!$status)
	{
		return $msg;
	}

	while( my ($key, $value) = each (%$cmdres_all_tasks))
	{
		if($key =~ m/list/ && $key =~ m/accounts/ &&
		   $key ne "cipux_task_list_user_accounts" &&
		   $key ne "cipux_task_list_role_accounts" &&
		   $key ne "cipux_task_list_skel_accounts")
		{
			my $cmdres;
			($status,$cmdres, $msg)=execCommand($key);
			if(!$status)
			{
				return $msg;
			}
			while( my ($gkey, $gvalue) = each (%$cmdres))
			{
				my $aref=$gvalue->{'memberUid'};
				my $exists='0';

				if( grep { $_ eq $user} @$aref)
				{
					$exists='1';
				}
				$str=$str."\n".$gkey.":$exists";
			}
		}
		if($key =~ m/list/ && $key =~ m/shares/)
		{
			my $cmdres;
			($status,$cmdres, $msg)=execCommand($key);
			if(!$status)
			{
				return $msg;
			}
			while( my ($gkey, $gvalue) = each (%$cmdres))
			{
				my $sharetype=(split("_",$key))[3];

				my $cmdshareres;
				($status,$cmdshareres, $msg)=execCommand("cipux_task_list_members_of_$sharetype"."_share", $gkey);
				if(!$status)
				{
					return $msg;
				}
				while( my ($gkey, $gvalue) = each (%$cmdshareres))
				{
					my $aref=$gvalue->{'memberUid'};
					my $exists='0';

					if( grep { $_ eq $user} @$aref)
					{
						$exists='1';
					}
					$str=$str."\n".$gkey.":$exists";
				}
			}
		}
	}
	$rpc->rpc_logout;
	return $str;
}

sub removeUsers
{
	my ($login, $password, $users) = @_;
	my ($status,$str)=getServerInfo($login, $password);
	if(!$status)
	{
		return $str;
	}
	my @ulist=split(";",$users);
	foreach(@ulist)
	{
		my ($tstatus,$res, $msg)=execCommand("cipux_task_destroy_user_account", $_);
		if(!$tstatus)
		{
			return $msg;
		}
	}
	$rpc->rpc_logout;
	return "OK";
}

sub removeGroups
{
	my ($login, $password, $groups) = @_;
	my ($status,$str)=getServerInfo($login, $password);
	if(!$status)
	{
		return $str;
	}

	my @rolegroups;
	my $msg;
	($status, $msg, @rolegroups)=getRoleGroups;
	if(!$status)
	{
		return $str;
	}

	my @glist=split(";",$groups);
	foreach(@glist)
	{
		my $grp=$_;
		if( grep { $_ eq $grp} @rolegroups)
		{
			#skip role group
		}
		else
		{
			my ($tstatus,$res, $msg)=execCommand("cipux_task_destroy_class_share", $grp);
			if(!$tstatus)
			{
				return $msg;
			}
		}
	}
	$rpc->rpc_logout;
	return "OK";
}


sub modifyGroup
{
	my ($login,$password,$group,@args)=@_;
	my $retmsg="OK";

	my ($status,$str)=getServerInfo($login, $password);
	if(!$status)
	{
		return $str;
	}
	return changeGroup($group,@args);
}

sub addGroup
{
	my ($login,$password,$group,@args)=@_;
	my $retmsg="OK";

	my ($status,$str)=getServerInfo($login, $password);
	if(!$status)
	{
		return $str;
	}
	my $type;
	foreach (@args)
	{
		my @cmd=split(":",$_);
		if(@cmd[0] eq "type")
		{
			$type=@cmd[1];
		}
	}
	my $res;
	my $msg;
	($status,$res, $msg)=execCommand('cipux_task_create_'.$type,$group);
	if(!$status)
	{
		return $msg;
	}
	return $retmsg;
}



sub changeGroup
{
	my ($group,@args)=@_;
	my @newusers;
	my @obsusers;
	my $type;
	my $password;
	my $str="OK";
	foreach (@args)
	{
		my @cmd=split(":",$_);
		if(@cmd[0] eq "password")
		{
			$password=@cmd[1];
		}
		if(@cmd[0] eq "type")
		{
			$type=@cmd[1];
		}
		if(@cmd[0] eq "newmembers")
		{
			@newusers=split(";",@cmd[1]);
		}
		if(@cmd[0] eq "obsoletemembers")
		{
			@obsusers=split(";",@cmd[1]);
		}
	}
	if($password && $type ne 'role')
	{
		my ($status,$cmdres, $msg)=execCommand('cipux_task_change_'.$type.'_password',$group,$password);
		if(!$status)
		{
			return $msg;
		}
	}
	foreach(@obsusers)
	{
		my ($status,$cmdres, $msg);
		if($type eq 'role' )
		{
			($status,$cmdres, $msg)=execCommand('cipux_task_remove_member_from_role_account',$group,$_);
		}
		else
		{
			($status,$cmdres, $msg)=execCommand('cipux_task_remove_member_from_'.$type,$group,$_);
		}
		if(!$status)
		{
			return $msg;
		}
	}
	foreach(@newusers)
	{
		my ($status,$cmdres, $msg);
		if($type eq 'role' )
		{
			($status,$cmdres, $msg)=execCommand('cipux_task_add_member_to_role_account',$group,$_);
		}
		else
		{
			($status,$cmdres, $msg)=execCommand('cipux_task_add_member_to_'.$type,$group,$_);
		}
		if(!$status)
		{
			return $msg;
		}
	}
	$rpc->rpc_logout;
	return $str;
}
